#include "TM1636.h"
#include <EEPROM.h> // nacteni knihoven

// promenne pro hlidani otevrene / zavrene lednicky
unsigned long ms = 0;
bool last_state = true; // zavreno = true
bool zavreno = true;

// promenne pro beeper
unsigned long previousMillis = 0;
const long interval = 200; // nastaveni rychlosti pipani
int beeperState = LOW;

TM1636 tm1636(7, 8); // pripojeni tm4966

// promenne pro switch
int buttonState = HIGH;      // the current buttonState of the output pin
int reading;           // the current reading from the input pin
int previous = LOW;    // the previous reading from the input pin
long time = 0;         // the last time the output pin was toggled
long debounce = 200;

// minimalni a maximalni namerene hodnoty
int minC;
int maxC;

String linka; // data ze seriove linky

void setup()
{
  Serial.begin(9600); // inicializace seriove linky
  pinMode(A1, INPUT); // fotorezistor >500 -> zavreno
  pinMode(6, OUTPUT); // buzzer
  pinMode(11, INPUT); // tlacitko MENU
  digitalWrite(11, HIGH); // pullup
  pinMode(10, INPUT); // tlacitko UP
  digitalWrite(10, HIGH); // pullup
  pinMode(9, INPUT); // tlacitko DOWN
  digitalWrite(9, HIGH); // pullup
  maxC = eepromIntRead(0); // nacti data z eepromky
  minC = eepromIntRead(2); // nacti data z eepromky
  tm1636.init(); // inicializace displeje
}
//--------------------------------------------------------------------------------------------------------
void loop()
{
  zjisti_stav_lednicky(); // zjisti jestli je lednice otevrena nebo ne
  cas_otevreni(); // ulozi cas hodin pri otevrene lednici
  Mswitch(); // nastaveni tlacitka jako prepinace
  max_min_temp(); // kontrola prekroceni teploty
  show_max_tmp(); // hlida stisk tlacitka UP a DOWN
  comm(); // nastaveni komunikace

  if (zavreno == false && millis() - ms > 30000) // kontroluj 30s otevreni, pokud je stale otevrena, tak piskej
    beep();

  if (zavreno == false) // pokud je otevrena, tak podle stavu prepinace zobraz teplotu
  {
    if (buttonState == HIGH)
      displayTemperature(get_tmpC(), true);
    else displayTemperature(get_tmpF(get_tmpC()), false);
  }

  else if (zavreno == true) // vsechno vypni
  {
    digitalWrite(6, LOW);
    tm1636.clearDisplay();
  }
}
//----------------------------------------------------------------------------------------------------------
void comm()
{
  if (Serial.available() > 0)
  {
    linka = Serial.readString();
    linka.toLowerCase();
    if (linka == "maxc")
      Serial.println(maxC);
    else if (linka == "minc")
      Serial.println(minC);
    else if (linka == "minf")
      Serial.println(get_tmpF(minC));
    else if (linka == "maxf")
      Serial.println(get_tmpF(maxC));
    else if (linka == "delete")
    {
      int temp = get_tmpC();
      eepromIntWrite(0, temp);
      maxC = eepromIntRead(0);
      eepromIntWrite(2, temp);
      minC = eepromIntRead(2);
      Serial.println("OK");
    }
    else if (linka == "allc")
    {
      Serial.println(minC);
      Serial.println(maxC);
    }
    else if (linka == "allf")
    {
      Serial.println(get_tmpF(minC));
      Serial.println(get_tmpF(maxC));
    }
    else Serial.println("ERROR");
  }
}

void show_max_tmp()
{
  if (digitalRead(9) == LOW)
  {
    displayTemperature(minC, true);
    delay(10000);
  }
  if (digitalRead(10) == LOW)
  {
    displayTemperature(maxC, true);
    delay(10000);
  }
}

void max_min_temp()
{
  int temp = get_tmpC();
  if (temp > maxC)
  {
    eepromIntWrite(0, temp);
    maxC = eepromIntRead(0); // na adrese 0-1 je ulozena max namerena hodnota
  }
  if (temp < minC)
  {
    eepromIntWrite(2, temp);
    minC = eepromIntRead(2); // na adrese 2-3 je ulozena min namerena hodnota
  }
}

void zjisti_stav_lednicky()
{
  if (analogRead(A1) > 200) // 200 = konstanta namerena na fotorezistoru
  {
    zavreno = false;
  }
  else zavreno = true;
}

void cas_otevreni() // magie s podminkama -> ulozi do promenne ms cas hodin, kdy se otevrela lednice
{
  if (zavreno == false && last_state == true)
  {
    ms = millis();
    last_state = false;
  }
  if (zavreno == true && last_state == false)
  {
    last_state = true;
  }
}

void beep() // zazvon
{
  unsigned long currentMillis = millis();

  if (currentMillis - previousMillis >= interval) {
    // save the last time you blinked the LED
    previousMillis = currentMillis;

    // if the LED is off turn it on and vice-versa:
    if (beeperState == LOW) {
      beeperState = HIGH;
    } else {
      beeperState = LOW;
    }

    // set the LED with the beeperState of the variable:
    digitalWrite(6, beeperState);
  }
}

int8_t get_tmpC() // nacti teplotu
{
  float temperature, resistance;
  int a;
  int B = 3975;
  a = analogRead(A0);
  resistance   = (float)(1023 - a) * 10000 / a;
  temperature  = 1 / (log(resistance / 10000) / B + 1 / 298.15) - 273.15;
  return (int8_t)temperature;
}

int get_tmpF(int celsius) // zjisti teplotu ve F
{
  return 1.8 * celsius + 32;
}

void displayTemperature(int8_t temperature, bool C) // vypis teplotu na displej; c == true, potom vypis C
{
  int8_t temp[4];
  if (temperature < 0)
  {
    temp[0] = 17;
    temperature = abs(temperature);
  }
  else if (temperature < 100)temp[0] = INDEX_BLANK;
  else temp[0] = temperature / 100;
  temperature %= 100;
  temp[1] = temperature / 10;
  temp[2] = temperature % 10;
  if (C == true)
    temp[3] = 12;           //index of 'C' for celsius degree symbol.
  else temp[3] = 15;
  tm1636.display(temp);
}

void Mswitch() // nastaveni prepinace
{
  reading = digitalRead(11);

  // if the input just went from LOW and HIGH and we've waited long enough
  // to ignore any noise on the circuit, toggle the output pin and remember
  // the time
  if (reading == HIGH && previous == LOW && millis() - time > debounce) {
    if (buttonState == HIGH)
      buttonState = LOW;
    else
      buttonState = HIGH;

    time = millis();
  }
  previous = reading;
}

void eepromIntWrite(int adresa, int hodnota) // nastaveni zapisu intu do eepromu
{
  byte byte1 = hodnota;
  byte byte2 = hodnota >> 8;
  EEPROM.write(adresa, byte1);
  EEPROM.write(adresa + 1, byte2);
}

int eepromIntRead(int adr) // nastaveni cteni intu z eepromu
{
  int hodnota = EEPROM.read(adr) + ((EEPROM.read(adr + 1)) << 8) ;
  return hodnota;
}

